package com.muyu.marketing.team.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.marketing.domain.ActivityTeamProductSkuInfo;
import com.muyu.marketing.domain.model.*;
import com.muyu.marketing.team.mapper.ActivityTeamProductSkuInfoMapper;
import com.muyu.marketing.team.service.ActivityTeamProductSkuInfoService;
import com.muyu.product.cache.ProjectSkuCache;
import com.muyu.product.domain.ProjectSkuInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;

@Service
public class ActivityTeamProductSkuInfoServiceImpl extends ServiceImpl<ActivityTeamProductSkuInfoMapper, ActivityTeamProductSkuInfo>
    implements ActivityTeamProductSkuInfoService {

    @Autowired
    private ProjectSkuCache projectSkuCache;

    /**
     * 通过团购活动ID获取团购中最优惠的价格
     *
     * @param teamId 团购ID
     * @return 优惠价格
     */
    @Override
    public TeamProductDiscountPriceModel getDiscountPrice(Long teamId) {
        List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = this.getActivityTeamProductSkuInfoByTeamId(teamId);

        // 优惠模型集合
        Optional<TeamProductDiscountPriceModel> discountPriceModelOptional = teamProductSkuInfoList.stream()
                .map(activityTeamProductSkuInfo -> {
                    ProjectSkuInfo projectSkuInfo = projectSkuCache.getData(activityTeamProductSkuInfo.getProductId(), activityTeamProductSkuInfo.getProductSku());
                    return TeamProductDiscountPriceModel.of(projectSkuInfo.getPrice(), activityTeamProductSkuInfo.getTeamPrice());
                }).min((o1, o2) -> Double.valueOf(o1.getDiscount() * 100 - o2.getDiscount() * 100).intValue());

        if (discountPriceModelOptional.isEmpty()){
            throw new ServiceException("团购活动下没有商品绑定");
        }
        return discountPriceModelOptional.get();
    }

    /**
     * 通过活动ID获取 剩余库存
     *
     * @param teamId 活动ID
     * @return 库存
     */
    @Override
    public TeamProductStockModel getStock(Long teamId) {
        List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = this.getActivityTeamProductSkuInfoByTeamId(teamId);
        return TeamProductStockModel.builder()
                .teamStock(teamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getTeamStock).reduce(0L, Long::sum))
                .remainStock(teamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getRemainStock).reduce(0L, Long::sum))
                .build();
    }

    /**
     * 添加团购商品SKU
     *
     * @param activityTeamProductSkuAddModelList 团购商品SKU添加模型集合
     */
    @Override
    public void batchSave(List<ActivityTeamProductSkuAddModel> activityTeamProductSkuAddModelList) {
        if (activityTeamProductSkuAddModelList == null || activityTeamProductSkuAddModelList.isEmpty()) {
            throw new ServiceException("入参添加商品规格模型，不可为空");
        }
        this.saveBatch(
                activityTeamProductSkuAddModelList.stream().map(ActivityTeamProductSkuInfo::modelBuild).toList()
        );
    }

    /**
     * 通过拼团活动ID获取拼团下商品SKU集合
     *
     * @param teamId 活动ID
     * @return 拼团商品SKU集合
     */
    @Override
    public List<ActivityTeamProductSkuModel> findListByTeamId(Long teamId) {
        LambdaQueryWrapper<ActivityTeamProductSkuInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ActivityTeamProductSkuInfo::getTeamId,teamId);
        List<ActivityTeamProductSkuInfo> list = this.list(queryWrapper);
        return list.stream().map(ActivityTeamProductSkuModel::FindBuild).toList();
//        return list.stream().map(activityTeamProductSkuInfo -> ActivityTeamProductSkuModel.FindBuild(activityTeamProductSkuInfo)).toList();
    }

    @Transactional
    @Override
    public boolean updateBath(List<ActivityTeamProductSkuReqModel> activityTeamProductSkuReqModelList) {
        return this.updateBatchById(activityTeamProductSkuReqModelList.stream().map(ActivityTeamProductSkuInfo::updateModelBuild).toList());
    }

    /**
     * 根据 业务模型 进行 团购商品修改
     *
     * @param activityTeamProductSkuUpdModel 修改业务模型
     * @return 修改结果
     */
    @Override
    public boolean update(ActivityTeamProductSkuUpdModel activityTeamProductSkuUpdModel) {
        // update set  where
        LambdaUpdateWrapper<ActivityTeamProductSkuInfo> updateWrapper = new LambdaUpdateWrapper<>();
        // 字段=值
        updateWrapper.set(ActivityTeamProductSkuInfo::getTeamStock,activityTeamProductSkuUpdModel.getTeamStock());
        updateWrapper.set(ActivityTeamProductSkuInfo::getRemainStock,activityTeamProductSkuUpdModel.getTeamStock());
        updateWrapper.set(ActivityTeamProductSkuInfo::getTeamPrice,activityTeamProductSkuUpdModel.getTeamPrice());
        // id=？
        updateWrapper.eq(ActivityTeamProductSkuInfo::getId,activityTeamProductSkuUpdModel.getId());
        return this.update(updateWrapper);
    }

    /**
     * 根据 业务模型 进行 团购商品批量修改
     *
     * @param activityTeamProductSkuUpdModelList 修改业务模型
     * @return 修改结果
     */
    @Override
    public boolean batchUpdate(List<ActivityTeamProductSkuUpdModel> activityTeamProductSkuUpdModelList) {
        return this.updateBatchById(
                activityTeamProductSkuUpdModelList.stream()
                        .map(ActivityTeamProductSkuInfo::updModelBuild)
                        .toList()
        );
    }
}
