package com.muyu.product.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.product.domain.AttributeInfo;
import com.muyu.product.domain.req.AttributeInfoQueryReq;
import com.muyu.product.domain.req.AttributeInfoSaveReq;
import com.muyu.product.domain.req.AttributeInfoEditReq;
import com.muyu.product.service.AttributeInfoService;
import com.muyu.common.core.web.page.TableDataInfo;

/**
 * 商品属性Controller
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Api(tags = "商品属性")
@RestController
@RequestMapping("/attribute")
public class AttributeInfoController extends BaseController {


    @Autowired
    private AttributeInfoService attributeInfoService;

    /**
     * 查询商品属性列表
     */
    @ApiOperation("获取商品属性列表")
    @RequiresPermissions("product:attribute:list")
    @GetMapping("/list")
    public Result<TableDataInfo<AttributeInfo>> list(AttributeInfoQueryReq attributeInfoQueryReq) {
        startPage();
        List<AttributeInfo> list = attributeInfoService.list(AttributeInfo.queryBuild(attributeInfoQueryReq));
        return getDataTable(list);
    }

    /**
     * 导出商品属性列表
     */
    @ApiOperation("导出商品属性列表")
    @RequiresPermissions("product:attribute:export")
    @Log(title = "商品属性", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, AttributeInfo attributeInfo) {
        List<AttributeInfo> list = attributeInfoService.list(attributeInfo);
        ExcelUtil<AttributeInfo> util = new ExcelUtil<AttributeInfo>(AttributeInfo.class);
        util.exportExcel(response, list, "商品属性数据");
    }

    /**
     * 获取商品属性详细信息
     */
    @ApiOperation("获取商品属性详细信息")
    @RequiresPermissions("product:attribute:query")
    @GetMapping(value = "/{id}")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class)
    public Result<AttributeInfo> getInfo(@PathVariable("id") Long id) {
        return Result.success(attributeInfoService.getById(id));
    }

    /**
     * 新增商品属性
     */
    @RequiresPermissions("product:attribute:add")
    @Log(title = "商品属性", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation("新增商品属性")
    public Result<String> add(@RequestBody AttributeInfoSaveReq attributeInfoSaveReq) {
        AttributeInfo attributeInfo = AttributeInfo.saveBuild(attributeInfoSaveReq);
        return attributeInfoService.save(attributeInfo) ? success(attributeInfo.getId()) : error();
    }

    /**
     * 修改商品属性
     */
    @RequiresPermissions("product:attribute:edit")
    @Log(title = "商品属性", businessType = BusinessType.UPDATE)
    @PutMapping("/{id}")
    @ApiOperation("修改商品属性")
    public Result<String> edit(@PathVariable Long id, @RequestBody AttributeInfoEditReq attributeInfoEditReq) {
        return toAjax(attributeInfoService.updateById(AttributeInfo.editBuild(id,attributeInfoEditReq)));
    }

    /**
     * 删除商品属性
     */
    @RequiresPermissions("product:attribute:remove")
    @Log(title = "商品属性", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    @ApiOperation("删除商品属性")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = String.class, example = "1,2,3,4")
    public Result<String> remove(@PathVariable List<Long> ids) {
        return toAjax(attributeInfoService.removeBatchByIds(ids));
    }
}
