package com.muyu.shop.cart.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.common.core.text.Convert;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.product.cache.ProjectInfoCache;
import com.muyu.product.cache.ProjectSkuCache;
import com.muyu.product.cache.ProjectSkuStockCache;
import com.muyu.product.cache.RuleInfoCache;
import com.muyu.product.cache.key.SkuStockKey;
import com.muyu.product.cache.model.RuleAttrCacheModel;
import com.muyu.product.cache.model.RuleCacheModel;
import com.muyu.product.domain.ProjectInfo;
import com.muyu.product.domain.ProjectSkuInfo;
import com.muyu.shop.cart.cache.CartCache;
import com.muyu.shop.cart.cache.key.CartHashKey;
import com.muyu.shop.cart.domain.model.CartSkuModel;
import com.muyu.shop.cart.domain.model.SkuRuleModel;
import com.muyu.shop.cart.domain.model.StatisticsCartModel;
import com.muyu.shop.cart.domain.req.CartInfoEditNumReq;
import com.muyu.shop.cart.domain.req.CartInfoIsSelectedUpdReq;
import com.muyu.shop.cart.domain.req.RemoveCartProjectReq;
import com.muyu.shop.cart.domain.resp.CartDetailResp;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.shop.cart.mapper.CartInfoMapper;
import com.muyu.shop.cart.domain.CartInfo;
import com.muyu.shop.cart.service.CartInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.transaction.annotation.Transactional;

/**
 * 购物车Service业务层处理
 *
 * @author DongZeLiang
 * @date 2024-03-29
 */
@Slf4j
@Service
public class CartInfoServiceImpl extends ServiceImpl<CartInfoMapper, CartInfo>  implements CartInfoService {

    @Autowired
    private CartCache cartCache;

    @Autowired
    private ProjectSkuCache projectSkuCache;

    @Autowired
    private ProjectInfoCache projectInfoCache;

    @Autowired
    private ProjectSkuStockCache projectSkuStockCache;

    @Autowired
    private RuleInfoCache ruleInfoCache;

    /**
     * 查询购物车列表
     *
     * @param cartInfo 购物车
     * @return 购物车
     */
    @Override
    public List<CartInfo> list(CartInfo cartInfo) {
        LambdaQueryWrapper<CartInfo> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(cartInfo.getProjectId())){
            queryWrapper.eq(CartInfo::getProjectId, cartInfo.getProjectId());
        }

        if (ObjUtils.notNull(cartInfo.getProjectSku())){
            queryWrapper.eq(CartInfo::getProjectSku, cartInfo.getProjectSku());
        }

        if (ObjUtils.notNull(cartInfo.getUserId())){
            queryWrapper.eq(CartInfo::getUserId, cartInfo.getUserId());
        }

        if (ObjUtils.notNull(cartInfo.getNum())){
            queryWrapper.eq(CartInfo::getNum, cartInfo.getNum());
        }

        if (ObjUtils.notNull(cartInfo.getIsSelected())){
            queryWrapper.eq(CartInfo::getIsSelected, cartInfo.getIsSelected());
        }

        return list(queryWrapper);
    }

    /**
     * 添加购物车
     *
     * @param cartInfo 购物车信息
     *
     * @return 是否添加成功
     */
    @Override
    public boolean add (CartInfo cartInfo) {
        Long userId = SecurityUtils.getUserId();
        CartHashKey cartHashKey = CartHashKey.builder()
                .projectId(cartInfo.getProjectId())
                .projectSku(cartInfo.getProjectSku())
                .build();
        SkuStockKey skuStockKey = SkuStockKey.builder()
                .projectId(cartInfo.getProjectId())
                .sku(cartInfo.getProjectSku())
                .build();
        Long skuStock = Convert.toLong(projectSkuStockCache.get(skuStockKey), -1L);
        if (cartCache.hasKey(userId, cartHashKey)){
            // 取出来修改
            CartInfo cartInfoCache = cartCache.get(userId, cartHashKey);
            cartInfoCache.setNum( cartInfoCache.getNum() + cartInfo.getNum());
            if (skuStock < cartInfoCache.getNum()){
                throw new ServiceException("当前库存不足");
            }
            LambdaUpdateWrapper<CartInfo> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.set(CartInfo::getNum, cartInfoCache.getNum());
            updateWrapper.eq(CartInfo::getId, cartInfoCache.getId());
            this.update(updateWrapper);
            this.cartCache.put(userId, cartHashKey, cartInfoCache);
        }else {
            // 存进去
            if (skuStock < cartInfo.getNum()){
                throw new ServiceException("当前库存不足");
            }
            cartInfo.setIsSelected("Y");
            cartInfo.setUserId(userId);
            cartInfo.setCreateBy(SecurityUtils.getUsername());
            cartInfo.setCreateTime(new Date());
            this.save(cartInfo);
            this.cartCache.put(userId, cartHashKey, cartInfo);
        }
        return true;
    }

    /**
     * 获取购物车详情
     *
     * @return 购物车详情
     */
    @Override
    public CartDetailResp detail () {
        Long userId = SecurityUtils.getUserId();
        List<CartInfo> cartInfoList = cartCache.getToList(userId);
        // CartInfo -> CartSkuModel
        List<CartSkuModel> cartSkuModelList = cartInfoList.stream()
                .map(cartInfo -> {
                    ProjectSkuInfo projectSkuInfo
                            = projectSkuCache.get(cartInfo.getProjectId(), cartInfo.getProjectSku());
                    ProjectInfo projectInfo
                            = projectInfoCache.get(cartInfo.getProjectId());
                    Long stock = projectSkuStockCache.get(
                            SkuStockKey.builder()
                                    .projectId(cartInfo.getProjectId())
                                    .sku(cartInfo.getProjectSku())
                                    .build()
                    );
                    // RuleCache
                    RuleCacheModel ruleInfoCacheModel = ruleInfoCache.getData(projectInfo.getRuleId());
                    List<RuleAttrCacheModel> ruleAttrModelList = ruleInfoCacheModel.getRuleAttrModelList();
                    List<SkuRuleModel> ruleModelList = new ArrayList<>();
                    String projectSku = cartInfo.getProjectSku();
                    String[] skuArr = projectSku.split("-");
                    for (int index = 0 ; index < skuArr.length ; index++) {
                        ruleModelList.add(
                                SkuRuleModel.builder()
                                        .ruleName(ruleAttrModelList.get(index).getAttrName())
                                        .ruleValue(skuArr[index])
                                        .build()
                        );
                    }
                    return CartSkuModel.builder()
                            .projectId(cartInfo.getProjectId())
                            .projectSku(cartInfo.getProjectSku())
                            .name(projectInfo.getName())
                            .image(projectSkuInfo.getImage())
                            .stock(stock)
                            .subtotal(projectSkuInfo.getPrice().multiply(new BigDecimal(cartInfo.getNum())))
                            .skuRuleList(ruleModelList)
                            .price(projectSkuInfo.getPrice())
                            .cartInfoId(cartInfo.getId())
                            .num(cartInfo.getNum())
                            .isSelected(cartInfo.getIsSelected())
                            .build();
                }).toList();


        StatisticsCartModel statisticsCartModel = StatisticsCartModel.builder()
                .total(
                        cartSkuModelList.stream()
                                .filter(cartSkuModel -> cartSkuModel.getStock() > 0)
                                .mapToLong(CartSkuModel::getNum)
                                .sum()
                )
                .selectTotal(
                        cartSkuModelList.stream()
                                .filter(cartSkuModel -> "Y".equals(cartSkuModel.getIsSelected()))
                                .filter(cartSkuModel -> cartSkuModel.getStock() > 0)
                                .mapToLong(CartSkuModel::getNum)
                                .sum()
                )
                .priceTotal(
                        cartSkuModelList.stream()
                                .filter(cartSkuModel -> "Y".equals(cartSkuModel.getIsSelected()))
                                .filter(cartSkuModel -> cartSkuModel.getStock() > 0)
                                .map(CartSkuModel::getSubtotal)
                                .reduce(BigDecimal.ZERO, BigDecimal::add)
                )
                .actualTotal(
                        cartSkuModelList.stream()
                                .filter(cartSkuModel -> "Y".equals(cartSkuModel.getIsSelected()))
                                .filter(cartSkuModel -> cartSkuModel.getStock() > 0)
                                .map(CartSkuModel::getSubtotal)
                                .reduce(BigDecimal.ZERO, BigDecimal::add)
                )
                .build();
        return CartDetailResp.builder()
                .cartSkuList(cartSkuModelList)
                .statisticsCart(statisticsCartModel)
                .build();
    }

    /**
     * 修改状态
     *
     * @param cartInfoIsSelectedUpdReqList
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void cartInfoIsSelected (List<CartInfoIsSelectedUpdReq> cartInfoIsSelectedUpdReqList) {
        for (CartInfoIsSelectedUpdReq cartInfoIsSelectedUpdReq : cartInfoIsSelectedUpdReqList) {
            LambdaUpdateWrapper<CartInfo> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.set(CartInfo::getIsSelected, cartInfoIsSelectedUpdReq.getIsSelected());
            updateWrapper.eq(CartInfo::getProjectId, cartInfoIsSelectedUpdReq.getProjectId());
            updateWrapper.eq(CartInfo::getProjectSku, cartInfoIsSelectedUpdReq.getProjectSku());
            this.update(updateWrapper);
            CartHashKey cartHashKey = CartHashKey.builder()
                    .projectId(cartInfoIsSelectedUpdReq.getProjectId())
                    .projectSku(cartInfoIsSelectedUpdReq.getProjectSku())
                    .build();
            Long userId = SecurityUtils.getUserId();
            CartInfo cartInfo = this.cartCache.get(userId, cartHashKey);
            cartInfo.setIsSelected(cartInfoIsSelectedUpdReq.getIsSelected());
            this.cartCache.put(userId, cartHashKey, cartInfo);
        }
    }

    @Override
    public void CartInfoEditNum (CartInfoEditNumReq cartInfoEditNumReq) {
        LambdaUpdateWrapper<CartInfo> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(CartInfo::getIsSelected, cartInfoEditNumReq.getNum());
        updateWrapper.eq(CartInfo::getProjectId, cartInfoEditNumReq.getProjectId());
        updateWrapper.eq(CartInfo::getProjectSku, cartInfoEditNumReq.getProjectSku());
        this.update(updateWrapper);
        Long userId = SecurityUtils.getUserId();
        CartHashKey cartHashKey = CartHashKey.builder()
                .projectId(cartInfoEditNumReq.getProjectId())
                .projectSku(cartInfoEditNumReq.getProjectSku())
                .build();
        CartInfo cartInfo = this.cartCache.get(userId, cartHashKey);
        cartInfo.setNum(cartInfoEditNumReq.getNum());
        this.cartCache.put(userId, cartHashKey, cartInfo);
    }

    /**
     * 删除
     *
     * @param removeCartProjectReqList
     *
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void removeByRemoveCartProjectList (List<RemoveCartProjectReq> removeCartProjectReqList) {
        this.removeByIds(
                removeCartProjectReqList.stream()
                        .map(RemoveCartProjectReq::getCartInfoId)
                        .toList()
        );
        Long userId = SecurityUtils.getUserId();
        removeCartProjectReqList.stream()
                .map(removeCartProjectReq -> CartHashKey.builder()
                        .projectId(removeCartProjectReq.getProjectId())
                        .projectSku(removeCartProjectReq.getProjectSku())
                        .build()
                ).forEach(hk -> this.cartCache.remove(userId, hk));
    }

}
